@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2OSupervisedAlgorithmWithFoldColumn import H2OSupervisedAlgorithmWithFoldColumn
 from ai.h2o.sparkling.ml.models.H2OGAMMOJOModel import H2OGAMMOJOModel
@@ -46,7 +46,6 @@ class H2OGAM(H2OGAMParams, H2OSupervisedAlgorithmWithFoldColumn, FamilyBasedH2OT
                  convertUnknownCategoricalLevelsToNa=False,
                  splitRatio=1.0,
                  withStageResults=False,
-                 withDetailedPredictionCol=True,
                  seed=-1,
                  family="AUTO",
                  tweedieVariancePower=0.0,
@@ -80,6 +79,8 @@ class H2OGAM(H2OGAMParams, H2OSupervisedAlgorithmWithFoldColumn, FamilyBasedH2OT
                  computePValues=False,
                  removeCollinearCols=False,
                  numKnots=None,
+                 splineOrders=None,
+                 splinesNonNegative=None,
                  scale=None,
                  bs=None,
                  keepGamCols=False,
