@@ -17,7 +17,7 @@
 
 from warnings import warn
 from pyspark import keyword_only
-from ai.h2o.sparkling import Initializer
+from ai.h2o.sparkling.Initializer import Initializer
 from ai.h2o.sparkling.ml.Utils import Utils
 from ai.h2o.sparkling.ml.algos.H2OGLM import H2OGLM
 
@@ -49,6 +49,7 @@ class H2OGLMRegressor(H2OGLM):
                  family="AUTO",
                  randomFamily=None,
                  tweedieVariancePower=0.0,
+                 dispersionLearningRate=0.5,
                  tweedieLinkPower=1.0,
                  theta=1.0E-10,
                  solver="AUTO",
@@ -68,10 +69,15 @@ class H2OGLMRegressor(H2OGLM):
                  gradientEpsilon=-1.0,
                  objReg=-1.0,
                  link="family_default",
+                 dispersionParameterMethod="pearson",
                  randomLink=None,
                  startval=None,
                  calcLike=False,
+                 generateVariableInflationFactors=False,
                  intercept=True,
+                 buildNullModel=False,
+                 fixDispersionParameter=False,
+                 initDispersionParameter=1.0,
                  HGLM=False,
                  prior=-1.0,
                  lambdaMinRatio=-1.0,
@@ -82,7 +88,11 @@ class H2OGLMRegressor(H2OGLM):
                  maxAfterBalanceSize=5.0,
                  maxConfusionMatrixSize=20,
                  computePValues=False,
+                 fixTweedieVariancePower=True,
                  removeCollinearCols=False,
+                 dispersionEpsilon=1.0E-4,
+                 tweedieEpsilon=8.0E-17,
+                 maxIterationsDispersion=3000,
                  generateScoringHistory=False,
                  modelId=None,
                  nfolds=0,
