@@ -64,6 +64,11 @@ class H2OGLMMOJOModel(H2OSupervisedMOJOModelParams, HasIgnoredColsOnMOJO):
         return value
 
 
+    def getDispersionLearningRate(self):
+        value = self._java_obj.getDispersionLearningRate()
+        return value
+
+
     def getTweedieLinkPower(self):
         value = self._java_obj.getTweedieLinkPower()
         return value
@@ -159,6 +164,11 @@ class H2OGLMMOJOModel(H2OSupervisedMOJOModelParams, HasIgnoredColsOnMOJO):
         return value
 
 
+    def getDispersionParameterMethod(self):
+        value = self._java_obj.getDispersionParameterMethod()
+        return value
+
+
     def getRandomLink(self):
         value = self._java_obj.getRandomLink()
         return H2OTypeConverters.scalaArrayToPythonArray(value)
@@ -174,11 +184,31 @@ class H2OGLMMOJOModel(H2OSupervisedMOJOModelParams, HasIgnoredColsOnMOJO):
         return value
 
 
+    def getGenerateVariableInflationFactors(self):
+        value = self._java_obj.getGenerateVariableInflationFactors()
+        return value
+
+
     def getIntercept(self):
         value = self._java_obj.getIntercept()
         return value
 
 
+    def getBuildNullModel(self):
+        value = self._java_obj.getBuildNullModel()
+        return value
+
+
+    def getFixDispersionParameter(self):
+        value = self._java_obj.getFixDispersionParameter()
+        return value
+
+
+    def getInitDispersionParameter(self):
+        value = self._java_obj.getInitDispersionParameter()
+        return value
+
+
     def getHGLM(self):
         value = self._java_obj.getHGLM()
         return value
@@ -229,11 +259,31 @@ class H2OGLMMOJOModel(H2OSupervisedMOJOModelParams, HasIgnoredColsOnMOJO):
         return value
 
 
+    def getFixTweedieVariancePower(self):
+        value = self._java_obj.getFixTweedieVariancePower()
+        return value
+
+
     def getRemoveCollinearCols(self):
         value = self._java_obj.getRemoveCollinearCols()
         return value
 
 
+    def getDispersionEpsilon(self):
+        value = self._java_obj.getDispersionEpsilon()
+        return value
+
+
+    def getTweedieEpsilon(self):
+        value = self._java_obj.getTweedieEpsilon()
+        return value
+
+
+    def getMaxIterationsDispersion(self):
+        value = self._java_obj.getMaxIterationsDispersion()
+        return value
+
+
     def getGenerateScoringHistory(self):
         value = self._java_obj.getGenerateScoringHistory()
         return value
@@ -328,4 +378,5 @@ class H2OGLMMOJOModel(H2OSupervisedMOJOModelParams, HasIgnoredColsOnMOJO):
         value = self._java_obj.getAucType()
         return value
 
+    # Outputs
 
